package apihandler

import (
	"context"
	"net/http"

	"git.agecem.com/agecem/agecem-org/apiresponse"
	"github.com/labstack/echo/v4"
	"github.com/minio/minio-go/v7"
)

// V1DocumentPOST permet d'ajouter un object dans un bucket, par multipart/form-data
func (h *V1Handler) V1DocumentPOST(c echo.Context) error {
	var response apiresponse.V1DocumentPOST

	bucket := c.Param("bucket")

	form_file, err := c.FormFile("document")
	if err != nil {
		response.StatusCode = http.StatusBadRequest
		response.Message = "Error during HandleV1DocumentCreate's echo#Context.FormFile"
		response.Error = err.Error()

		return c.JSON(response.StatusCode, response)
	}

	allowed := false
	for bucket_allowed := range h.Config.Server.Documents.Buckets {
		if bucket == bucket_allowed {
			allowed = true
		}
	}

	if !allowed {
		return c.JSON(apiresponse.NotFoundResponse())
	}

	ctx, cancel := context.WithCancel(context.Background())

	defer cancel()

	src, err := form_file.Open()
	if err != nil {
		response.StatusCode = http.StatusBadRequest
		response.Message = "Error during form_file.Open()"
		response.Error = err.Error()

		return c.JSON(response.StatusCode, response)
	}
	defer src.Close()

	info, err := h.MediaClient.MinioClient.PutObject(ctx, bucket, form_file.Filename, src, form_file.Size, minio.PutObjectOptions{
		ContentType: form_file.Header.Get("Content-Type"),
	})
	if err != nil {
		response.StatusCode = http.StatusInternalServerError
		response.Message = "Error during minio#PutObject"
		//response.Error = err.Error()

		return c.JSON(response.StatusCode, response)
	}

	response.StatusCode = http.StatusOK
	response.Message = "ok"
	response.Data.Bucket = info.Bucket
	response.Data.Key = info.Key
	response.Data.Size = info.Size

	return c.JSON(response.StatusCode, response)
}

// V1DocumentGET permet de lire le contenu d'un fichier et protentiellement de le télécharger
func (h *V1Handler) V1DocumentGET(c echo.Context) error {
	bucket := c.Param("bucket")
	document := c.Param("document")

	allowed := false
	for bucket_allowed := range h.Config.Server.Documents.Buckets {
		if bucket == bucket_allowed {
			allowed = true
		}
	}

	if !allowed {
		return c.JSON(apiresponse.NotFoundResponse())
	}

	ctx, cancel := context.WithCancel(context.Background())

	defer cancel()

	bucket_exists, err := h.MediaClient.MinioClient.BucketExists(ctx, bucket)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, "Error during minio#BucketExists")
	}

	if !bucket_exists {
		return c.JSON(apiresponse.NotFoundResponse())
	}

	document_info, err := h.MediaClient.MinioClient.StatObject(ctx, bucket, document, minio.StatObjectOptions{})

	if err != nil {
		if err.Error() == "The specified key does not exist." {

			return c.JSON(apiresponse.NotFoundResponse())
		}

		return c.JSON(http.StatusInternalServerError, map[string]interface{}{
			"message": "Error during minio#StatObject",
		})
	}

	_ = document_info

	document_object, err := h.MediaClient.MinioClient.GetObject(ctx, bucket, document, minio.GetObjectOptions{})
	if err != nil {
		return c.JSON(http.StatusInternalServerError, map[string]string{
			"message": "Error during minio#GetObject",
		})
	}

	defer document_object.Close()

	return c.Stream(http.StatusOK, document_info.ContentType, document_object)
}

// V1DocumentPUT permet de mettre à jour certains champs d'un object, comme le Content-Type ou le Filename
func (h *V1Handler) V1DocumentPUT(c echo.Context) error {
	return c.JSON(apiresponse.NotImplementedResponse())
}

// V1DocumentDELETE permet de supprimer un object
func (h *V1Handler) V1DocumentDELETE(c echo.Context) error {
	bucket := c.Param("bucket")
	document := c.Param("document")

	allowed := false
	for bucket_allowed := range h.Config.Server.Documents.Buckets {
		if bucket == bucket_allowed {
			allowed = true
		}
	}

	if !allowed {
		return c.JSON(apiresponse.NotFoundResponse())
	}

	ctx, cancel := context.WithCancel(context.Background())

	defer cancel()

	bucket_exists, err := h.MediaClient.MinioClient.BucketExists(ctx, bucket)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, "Error during minio#BucketExists")
	}

	if !bucket_exists {
		return c.JSON(apiresponse.NotFoundResponse())
	}

	document_info, err := h.MediaClient.MinioClient.StatObject(ctx, bucket, document, minio.StatObjectOptions{})
	if err != nil {
		if err.Error() == "The specified key does not exist." {

			return c.JSON(apiresponse.NotFoundResponse())
		}

		return c.JSON(http.StatusInternalServerError, map[string]interface{}{
			"message": "Error during minio#StatObject",
		})
	}

	//TODO Add error validation
	_ = document_info

	err = h.MediaClient.MinioClient.RemoveObject(ctx, bucket, document, minio.RemoveObjectOptions{})
	if err != nil {
		return c.JSON(http.StatusInternalServerError, map[string]string{
			"message": "Error during minio#RemoveObject",
		})
	}

	return c.JSON(http.StatusOK, map[string]string{
		"message": "Document deleted",
	})
}
